﻿using SpreadSheetSpaceSDK;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace SDKExample_CSharp
{
    class Program
    {
        /**
         * The SpreadSheetSpace server url (i.e. https://www.spreadsheetspace.net)
         */
        static String server = "";

        /**
         * The token of the SpreadSheetSpace user that will run this program
         */
        static String userToken = "";

        /**
         * A couple of recipients (email addresses) that will receive the view created by this program
         */
        static String recipient1 = "";
        static String recipient2 = "";

        /**
         * If set to true this program will create a private view
         */
        static Boolean createPrivate = false;

        /**
         * If set to true this program will create a public view
         */
        static Boolean createPublic = true;
        

        /*
         * Sleeps are included only to give user time to:
         * - import the newly created view in Excel
         * - view script data
	     */
        static int sleepTime = 30 * 1000;
        static Boolean sleep = true;

        static void Main(string[] args)
        {
            try
            {
                // Initialize the SpreadSheetSpace SDK
                Sdk sdk = new Sdk(server, userToken);

                if (createPrivate)
                {
                    privateViewExample(sdk);
                }

                if (createPublic)
                {
                    publicViewExample(sdk);
                }

            }
            catch (Exception e)
            {
                Console.WriteLine("Exception", e);
            }

        }

        static public void privateViewExample(Sdk sdk)
        {
            try
            {
                String view_id = "";
                String view_server = "";

                Console.WriteLine("Key pair generation....");
                GenerateKeyDescriptor generateKeyDescriptor = sdk.GenerateKey();
                String privateKey = generateKeyDescriptor.PrivateKey;
                Console.WriteLine("Private key:");
                Console.WriteLine(privateKey);
                Console.WriteLine("The key has been generated and saved");

                int cols = 2;
                int rows = 2;
                String[,] table = { { "A", "B" }, { "C", "D" } };

                List<String> recipients = new List<String>();
                recipients.Add(recipient1);
                String excel_template = Path.Combine(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).Parent.Parent.FullName, "template1.xlsx");

                Console.WriteLine("\nCreating private view....");
                ViewDescriptor viewDescriptor = sdk.createPrivateView("Test Range", recipients, table, excel_template, false, false, rows, cols, true);

                if (viewDescriptor.StatusCode == 200) //OK
                {
                    Console.WriteLine("The private view has been created");
                    view_id = viewDescriptor.ViewId;
                    view_server = viewDescriptor.ViewServer;

                    Console.WriteLine("\nRetrieving view values....");
                    ValuesViewDescriptor valuesViewDescriptor = sdk.getValues(view_id, view_server, privateKey, null);
                    Object[,] values = valuesViewDescriptor.Values;

                    int row = values.GetLength(0);
                    int col = values.GetLength(1);
                    for (int i = 1; i <= row; i++)
                    {
                        for (int j = 1; j <= col; j++)
                        {
                            Console.WriteLine("(" + i + ", " + j + "): " + values[i, j]);
                        }
                    }

                    table = new String[,] { { "1", "2" }, { "3", "4" } };

                    excel_template = Path.Combine(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).Parent.Parent.FullName, "template2.xlsx");

                    if (sleep)
                    {
                        Console.WriteLine("Sleeping....");
                        Thread.Sleep(sleepTime);
                    }

                    Console.WriteLine("\nAdding recipients...." + recipient2);
                    List<String> addListRecipients = new List<String>();
                    addListRecipients.Add(recipient2);

                    ChangeRecipientDescriptor changeRecipientAddDescriptor = sdk.addRecipients(view_id, view_server, addListRecipients);
                    if (changeRecipientAddDescriptor.StatusCode == 200) //OK
                    {
                        Console.WriteLine("The recipients have been added.");
                    }
                    else
                    {
                        Console.WriteLine("Error adding recipients: ");
                        for (int i = 0; i < changeRecipientAddDescriptor.Message.Count(); i++)
                        {
                            Console.WriteLine(changeRecipientAddDescriptor.Message.ElementAt(i));
                        }
                    }

                    Console.WriteLine("\nUpdating private view....");
                    ViewDescriptor viewDescriptorUpdate;
                    int attempts = 0;
                    /*
                     * The following do-while block is necessary because it could happen that the view we are trying to update might have been already updated from somewhere else (i.e. from Excel).
                     * In this way we check if the next available sequence number we have is currently the last or if we need to retrieve the last available from the server.
                     */
                    do
                    {
                        viewDescriptorUpdate = sdk.updateView(viewDescriptor, table, excel_template, true);

                        if (viewDescriptorUpdate.StatusCode == 403) //WRONG_NEXT_NUMBER
                        {
                            viewDescriptor = new ViewDescriptor(view_id, view_server, viewDescriptorUpdate.NextAvailableSequenceNumber);
                        }
                        attempts++;
                    } while (viewDescriptorUpdate.StatusCode != 200 && attempts < 5);


                    if (viewDescriptorUpdate.StatusCode == 200) //OK
                    {
                        Console.WriteLine("The private view has been updated");

                        ValuesViewDescriptor valuesViewDescriptorUpdate = sdk.getValues(view_id, view_server, privateKey, null);
                        values = valuesViewDescriptorUpdate.Values;

                        row = values.GetLength(0);
                        col = values.GetLength(1);
                        for (int i = 1; i <= row; i++)
                        {
                            for (int j = 1; j <= col; j++)
                            {
                                Console.WriteLine("(" + i + ", " + j + "): " + values[i, j]);
                            }
                        }

                        if (sleep)
                        {
                            Console.WriteLine("Sleeping....");
                            Thread.Sleep(sleepTime);
                        }

                        Console.WriteLine("\nRemoving recipients...." + recipient2);
                        List<String> deleteListRecipients = new List<String>();
                        deleteListRecipients.Add(recipient2);

                        ChangeRecipientDescriptor changeRecipientDeleteDescriptor = sdk.removeRecipients(view_id, view_server, deleteListRecipients);
                        if (changeRecipientDeleteDescriptor.StatusCode == 200) //OK
                        {
                            Console.WriteLine("The recipients have been removed.");
                        }
                        else
                        {
                            Console.WriteLine("Error removing recipients:");
                            for (int i = 0; i < changeRecipientDeleteDescriptor.Message.Count(); i++)
                            {
                                Console.WriteLine(changeRecipientDeleteDescriptor.Message.ElementAt(i));
                            }
                        }

                        if (sleep)
                        {
                            Console.WriteLine("Sleeping....");
                            Thread.Sleep(sleepTime);
                        }

                        Console.WriteLine("\nDeleting the private view...");
                        DeleteViewDescriptor viewDescriptorDelete = sdk.deleteView(view_id, view_server);
                        if (viewDescriptorDelete.StatusCode == 200) //OK
                        {
                            Console.WriteLine("The private view has been deleted");
                        }
                        else
                        {
                            Console.WriteLine("Error deleting the private view:");
                            Console.WriteLine(viewDescriptorDelete.Message);
                            for (int i = 0; i < viewDescriptorDelete.Message.Count(); i++)
                            {
                                Console.WriteLine(viewDescriptorDelete.Message.ElementAt(i));
                            }
                        }
                    }
                    else
                    {
                        Console.WriteLine("Error updating the private view:");
                        Console.WriteLine(viewDescriptorUpdate.Message);
                    }
                }
                else
                {
                    Console.WriteLine("Error creating the private view:");
                    Console.WriteLine(viewDescriptor.Message);
                }

                Console.WriteLine("\nRetrieving contacts from addressbook...");
                AddressbookDescriptor addressBookDescriptor = sdk.getAddressBook();
                if (addressBookDescriptor.StatusCode == 200) //OK
                {
                    Console.WriteLine("Contacts retrieved:");
                    Contact[] contacts = addressBookDescriptor.ListContact;

                    for (int i = 0; i < contacts.Count(); i++)
                    {
                        Console.WriteLine(contacts[i].firstName + ", " + contacts[i].lastName + " - " + contacts[i].emailAddress);
                    }
                }
                else
                {
                    Console.WriteLine("Error retrieving contacts from addressbook.");
                }

                Console.WriteLine("\nRetrieving owned views...");
                ListViewDescriptor listOwnedDescriptor = sdk.getOwnedView();
                if (listOwnedDescriptor.StatusCode == 200) //OK
                {
                    Console.WriteLine("Owned views retrieved:");
                    for (int i = 0; i < listOwnedDescriptor.ListView.Count(); i++)
                    {
                        Console.WriteLine(listOwnedDescriptor.ListView.ElementAt(i).description + " - " + listOwnedDescriptor.ListView.ElementAt(i).owner);
                    }
                }
                else
                {
                    Console.WriteLine("Error retrieving owned views.");
                }

                Console.WriteLine("\nRetrieving inbox views...");
                ListViewDescriptor listInboxDescriptor = sdk.getInboxView();
                if (listInboxDescriptor.StatusCode == 200) //OK
                {
                    Console.WriteLine("Inbox views retrieved:");
                    for (int i = 0; i < listInboxDescriptor.ListView.Count(); i++)
                    {
                        Console.WriteLine(listInboxDescriptor.ListView.ElementAt(i).description + " - " + listInboxDescriptor.ListView.ElementAt(i).owner);
                    }
                }
                else
                {
                    Console.WriteLine("Error retrieving inbox views.");
                }

            }
            catch (Exception e) {
                Console.WriteLine("Exception creating private view", e);
		    }

        }

        static public void publicViewExample(Sdk sdk)
        {
            try
            {
                String view_id_public = "";
                String view_server_public = "";

                String excel_template = Path.Combine(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).Parent.Parent.FullName, "template1.xlsx");

                int rows = 2;
                int cols = 2;

                String[,] table = { { "A", "B" }, { "C", "D" } };

                Console.WriteLine("\nCreating public view....");
                ViewDescriptor viewDescriptorPublic = sdk.createPublicView("Public Range", table, excel_template, false, false, rows, cols);

                if (viewDescriptorPublic.StatusCode == 200) //OK
                {
                    Console.WriteLine("Public view has been created");
                    view_id_public = viewDescriptorPublic.ViewId;
                    view_server_public = viewDescriptorPublic.ViewServer;

                    Console.WriteLine("\nRetrieving view values....");
                    ValuesViewDescriptor valuesViewDescriptorPublic = sdk.getValues(view_id_public, view_server_public, "", null);
                    Object[,] values = valuesViewDescriptorPublic.Values;

                    int row = values.GetLength(0);
                    int col = values.GetLength(1);
                    for (int i = 1; i <= row; i++)
                    {
                        for (int j = 1; j <= col; j++)
                        {
                            Console.WriteLine("(" + i + ", " + j + "): " + values[i, j]);
                        }
                    }

                    Console.WriteLine("\nPublishing the public view....");
                    ViewDescriptor viewDescriptorPublish = sdk.publishPublicView(view_id_public, view_server_public, "Public view description", "This is only a sample view...");

                    if (viewDescriptorPublish.StatusCode == 200) //OK
                    {
                        Console.WriteLine("Public view successfully published.");

                        if (sleep)
                        {
                            Console.WriteLine("Sleeping....");
                            Thread.Sleep(sleepTime);
                        }

                        table = new String[,] { { "1", "2" }, { "3", "4" } };

                        excel_template = Path.Combine(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).Parent.Parent.FullName, "template2.xlsx");

                        Console.WriteLine("\nUpdating public view....");
                        ViewDescriptor viewDescriptorUpdatePublic;
                        int attempts = 0;
                        /*
                         * The following do-while block is necessary because it could happen that the view we are trying to update might have been already updated from somewhere else (i.e. from Excel).
                         * In this way we check if the next available sequence number we have is currently the last or if we need to retrieve the last available from the server.
                         */
                        do
                        {
                            viewDescriptorUpdatePublic = sdk.updateView(viewDescriptorPublic, table, excel_template, false);

                            if (viewDescriptorUpdatePublic.StatusCode == 403) //WRONG_NEXT_NUMBER
                            {
                                viewDescriptorPublic = new ViewDescriptor(view_id_public, view_server_public, viewDescriptorUpdatePublic.NextAvailableSequenceNumber);
                            }
                            attempts++;
                        } while (viewDescriptorUpdatePublic.StatusCode != 200 && attempts < 5);

                        if (viewDescriptorUpdatePublic.StatusCode == 200) //OK
                        {
                            Console.WriteLine("The public view has been updated");

                            ValuesViewDescriptor valuesViewDescriptorUpdate = sdk.getValues(view_id_public, view_server_public, "", null);
                            values = valuesViewDescriptorUpdate.Values;

                            row = values.GetLength(0);
                            col = values.GetLength(1);
                            for (int i = 1; i <= row; i++)
                            {
                                for (int j = 1; j <= col; j++)
                                {
                                    Console.WriteLine("(" + i + ", " + j + "): " + values[i, j]);
                                }
                            }

                            if (sleep)
                            {
                                Console.WriteLine("Sleeping....");
                                Thread.Sleep(sleepTime);
                            }

                        }
                        else
                        {
                            Console.WriteLine("Error updating the public view:");
                            Console.WriteLine(viewDescriptorUpdatePublic.Message);
                        }

                        Console.WriteLine("\nDeleting public view...");
                        DeleteViewDescriptor viewDescriptorDelete = sdk.deleteView(view_id_public, view_server_public);
                        if (viewDescriptorDelete.StatusCode == 200) //OK
                        {
                            Console.WriteLine("The public view has been deleted");
                        }
                        else
                        {
                            Console.WriteLine("Error deleting the public view:");
                            Console.WriteLine(viewDescriptorDelete.Message);
                        }
                    }
                    else
                    {
                        Console.WriteLine("Error publishing the public view:");
                        Console.WriteLine(viewDescriptorPublish.Message);
                    }
                }
                else
                {
                    Console.WriteLine("Error creating the public view:");
                    Console.WriteLine(viewDescriptorPublic.Message);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception creating public view", e);
            }

        }
    }
}
